#!/bin/sh

##############################################################################
# ADSL Connection Tester v0.1                                                #
# (C)2002 Lasse Danielsen (lasse@danielsen.tk)                               #
#                                                                            #
# Program is released as mailware - if you use it, send me an email.         #
# The author will not stand responsible for any use/misuse of this program.  #
# If you suddenly find youself with an empty harddrive and a burning ADSL-   #
# modem, it's your own fault. You have been warned!                          #
#                                                                            #
# This program has been tested on OpenBSD *only* but should work on diff.    #
# platforms with little or no modifications.                                 #
#                                                                            #
# Version History:                                                           #
# - July 29th, 2002: first release.                                          #
#                                                                            #
##############################################################################

##########
# Config #
##########

# Interface Used (normally tun0)
Interface="tun0"

# Set this to X in "tunX" above (normally 0)
InterfaceNumber="0"

# The physical NIC-name of your external NIC (xl0, ep0, etc)
hwnic="xl0"
# Number of times the program needs to be run before we roll up the sleeve and
# call this an emergency, ie resetting the ppp-tunnel.
MaxFailed=2

# Number of seconds to wait after a reset of the ppp-tunnel before checking
# (pinging) the connection again.
Wait=15

# This variable controls the number of Waits before restarting the ppp-tunnel
# again. This is to provide some time for the tunnel to be established before
# tearing it down and rebuilding it.
MaxWait="5"

# Should the program keep quiet if all is a-okay? 1 for yes, 0 for no. It is
# advisable to put this on, else your logs will get mighty big.
QuietMode="1"

# IP's to ping - make sure they all normally accept ICMP (ping) requests.
# Good ip's are gateways, DNS-servers, ISP's webpages etc.
Ips="193.212.1.10 193.69.165.20"

# If you want a mail sent when the connection has been reset, enter a mail-
# address here. If not, set to "".
MailAddress="psxc@psxc.com lasse@danielsen.tk"

#########################################
# System settings - edit if you need to #
#########################################
# Set this to your system's run dir - normally "/var/run"
RunDir="/var/run"

# Set this to your temp-dir - normally "/tmp"
Tmp="/tmp"

# Set this to the path/name of the file we should use as log. Normally
# "/var/log/messages" is a good place, unless you want to use a separate log.
Log="/var/log/adsl"

# Path variable - make sure all commands are availible
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin

##############################################################################
#                                                                            #
#                         No need to edit below                              #
#                                                                            #
##############################################################################

# A few needed variables
########################
Failed=0
TempFailed=0
Noop=`eval echo $0 | sed "s|/| |g" | wc -w`
Noop=`expr $Noop \+ 1`
NameRunning="`echo $0 | cut -d"/" -f $Noop`"

##############################################################################
# Configure some functions                                                   #
##############################################################################

# Mailsupport
#############
function Mail {
  MyIp="`/sbin/ifconfig $Interface | grep inet | cut -f 2 -d':' | cut -f 2 -d' ' | cut -d. -f-4 -s`"
  MyName="`hostname`"
  cat << EOM > $Tmp/$NameRunning.mail
ADSL connection was reset.
New IP for $MyName is $MyIp
EOM
 if [ "$MailAddress" = "" ]; then
  Noop=1
 else
  echo -n "`Date` Sending mail to the sysop(s) .. " >> $Log
  mail -s "IP-change for $MyName" $MailAddress < $Tmp/$NameRunning.mail 2>>$Log
  if [ "$?" = "0" ]; then
   echo "succeeded." >> $Log
  else
   echo "failed!" >> $Log
  fi
 fi
 echo "`Date` New IP is $MyIp." >> $Log
}

# Give date, servername, program-name and PID
#############################################
function Date {
 echo -n "`date "+%b %e %H:%M:%S"` `hostname -s` $NameRunning[`cat $RunDir/$NameRunning.pid`]:"
}

# Kill PPP tunnel
#################
function KillPPP {
 echo "`Date` Will try to kill all instances of ppp." >> $Log
 ps -x -o pid,command | grep -v `cat $RunDir/$NameRunning.pid` > $Tmp/$NameRunning.output
 PidList="`eval cat $Tmp/$NameRunning.output | sed -n '/ppp/p' | awk '{ print $1 }'`"
 if [ -z `echo $PidList` ]; then
  echo "`Date` No instances of ppp found. Looks like it's not running." >> $Log
 else
  kill -9 `echo $PidList` 2>/dev/null
  echo "`Date` ppp killed." >> $Log
 fi
 rm -f $Tmp/$NameRunning.output 2>/dev/null
 rm -f $RunDir/$Interface.pid 2>/dev/null
}

# Check to see if already running
#################################
function CheckRunning {
 if [ -e $RunDir/$NameRunning.pid ]; then
  TempPid="`cat $RunDir/$NameRunning.pid`"
  if [ ! -z "`ps -x -o pid,command | grep $TempPid | { read pid crap; echo $pid; }`" ]; then
   echo "`Date` Another instance of this program is already running - exiting." >> $Log
   exit 0
  else
   echo "`ps -x -o pid,command | grep $NameRunning | { read pid crap; echo $pid; }`" > $RunDir/$NameRunning.pid
  fi
 else
  echo "`ps -x -o pid,command | grep $NameRunning | { read pid crap; echo $pid; }`" > $RunDir/$NameRunning.pid
 fi
}

# Check for connectivity
########################
function Connectivity {
 if [ "$QuietMode" = "0" ]; then
  echo -n "`Date` ping: " >> $Log
 else
  Noop=1
 fi
 for Ip in $Ips; do
  ping -c 1 $Ip 2>/dev/null >/dev/null
  if [ "$?" != "0" ]; then
   Failed="`expr $Failed \+ 1`"
   if [ "$QuietMode" = "0" ]; then
    echo -n "$Ip .. " >> $Log
   else
    Noop=1
   fi
  else
   if [ "$QuietMode" = "0" ]; then
    echo "$Ip is up." >> $Log
    if [ "$Status" = "Down" ]; then
     Status="Back"
    else
     Noop=1
    fi
   else
    Noop=1
   fi
   if [ "$Status" = "Back" ]; then
    echo "`Date` Line is back up. All is okay." >> $Log
    Mail
   else
    Noop=1
   fi
   rm -f $RunDir/$NameRunning.pid
   rm -f $RunDir/$NameRunning.count
   exit 0
  fi
 done
 if [ "$QuietMode" = "0" ]; then
  echo "Down." >> $Log
 else
  Noop=1
 fi
 touch $RunDir/$NameRunning.count
 if [ -z `cat $RunDir/$NameRunning.count` ]; then
  echo "1" > $RunDir/$NameRunning.count
  TempFailed="1"
 else
  TempFailed="`cat $RunDir/$NameRunning.count`"
  TempFailed="`expr $TempFailed \+ 1`"
  echo $TempFailed > $RunDir/$NameRunning.count
 fi
}

# Check to see if we should reset the ADSL connection
#####################################################
function ADSLReset {
 TempFailed="`cat $RunDir/$NameRunning.count`"
 if [ $TempFailed -lt $MaxFailed ]; then
  echo "`Date` No connection - This was try $TempFailed of $MaxFailed." >> $Log
  echo "`Date` Will wait and check again on the next run." >> $Log
  echo "$TempFailed" > $RunDir/$NameRunning.count
  if [ "$Status" = "Up" ]; then
   rm -f $RunDir/$NameRunning.pid 2>/dev/null
   exit 0
  else
   TempFailed="`cat $RunDir/$NameRunning.count`"
   TempFailed="`expr $TempFailed \+ 1`"
   echo $TempFailed > $RunDir/$NameRunning.count
  fi
 else
  echo "`Date` No connection - This was try $TempFailed of $MaxFailed." >> $Log
  echo "`Date` Will now re-establish the connection." >> $Log
 fi
}

# Kill the device/tunnel
########################
function KillTunnel {
 route delete default 2>/dev/null >/dev/null
 echo -n "`Date` Default route removed" >> $Log
 if [ -e $RunDir/$Interface.pid ]; then
  TempPid="`cat $RunDir/$Interface.pid`"
  if [ ! -z "`ps -wuax | grep $TempPid | { read crap pid morecrap; echo $pid; }`" ]; then
   kill -9 `cat $RunDir/$Interface.pid` 2>/dev/null
   rm -f $RunDir/$Interface.pid 2>/dev/null
   ifconfig $hwnic down
   sleep 5
   ifconfig $hwnic up
   echo " and $Interface killed." >> $Log
  else
  echo "." >> $Log
  echo "`Date` Hm. Can't find the pid for $Interface." >> $Log
   KillPPP
  fi
 else
  echo "." >> $Log
  KillPPP
 fi
}

# Restart the connection
########################
function ADSLRestart {
 ppp -unit$InterfaceNumber -ddial pppoe 2>/dev/null >/dev/null
 sleep 5
 echo "`Date` (Re)started the ppp tunnel." >> $Log
}

##############################################################################
# "Main" program                                                             #
##############################################################################

# Check to see if all is okay
#############################
Status="Up"
CheckRunning
Connectivity
ADSLReset
Status="Down"
while [ $Status = "Down" ]; do
 KillTunnel
 ADSLRestart
 echo "0" > $RunDir/$NameRunning.count
 while [ `cat $RunDir/$NameRunning.count` -le $MaxWait ]; do
  echo "`Date` Waiting $Wait seconds before testing connection again." >> $Log
  sleep $Wait
  QuietMode="0"
  Connectivity
  TempFailed="`cat $RunDir/$NameRunning.count`"
  TempFailed="`expr $TempFailed \+ 1`"
  echo "$TempFailed" > $RunDir/$NameRunning.count
  echo "`Date` No connection." >> $Log
 done
 ADSLReset
done
rm -f $RunDir/$NameRunning.pid 2>/dev/null
exit 0

